/*
 
 _________________
     MSet+.pov
 2011 Samuel Benge
 
 A gallery of nine true-color fractals, with the option to write your own
 if you're feeling brave.
 
 The Fractal Types:
  1 - classic Mandelbrot (MSet)
  2 - Julia
  3 - Glynn fractal
  4 - Burning Ship (mostly horrible)
  5 - MSet variation 1
  6 - MSet variation 3
  7 - half Burning Ship/half MSet
  8 - Burning Ship/ Mandelbrot hybrid
  9 - Burning Ship/ Mandelbrot hybrid variation 2
  
  Additional options are available for some of these. Check below.
 
  
 The Coloring Methods:
  1 - standard coloring by iteration
  2 - normalized smoothing (iterations will affect this, so be warned)
 
 
 Palette Support
 
  Palettes in the form of image files can be used. Just set 'UsePalette' to true.
  See the palettes folder for details, and also the palette section below.
 
 
 Navigation
  
  Navigation is performed by calling the Zoom_Region() macro one or more times
  inside the Navigation() macro.
  
  Zoom_Region(Position, Scale, Rotation, DoZoom)
    *Position = position of zoom box, a 2D vector
    *Scale    = scale of zoom box, a float
    *Rotation = rotation of zoom box, a float
    *DoZoom   = zoom mode, bool
      false = don't zoom, but display the zoom box
      true  = zoom into region, and don't display the box
    
    Basically, what you do is adjust the zoom region with 'DoZoom' set to false.
    When you find a good spot, set 'DoZoom' to true and type up a new Zoom_Region
    instance. The effects are additive with every new zoom region, so you just
    repeat the process.
    
    The navigational overlay (grid, numbers, box) will disappear when the last 
    region has been zoomed into.
    
  
  Aspect Correction
  
    There's no need to worry about the render window size, as everything is mostly
    taken care of. However, changing the render aspect /after/ positioning/zooming
    (eg. from 320x240 to 400x400) is not recommended, as you will lose your place.
    Saveable parameters might overcome this limitation...
    
  
  A Final Note
  
    You might want to explore fractal type 8, as there are some very interesting
    things to find there...
  
  Have Fun :)
    
  
*/
#version 3.7;

// -=-=-=-=-=-=-=- Settings -=-=-=-=-=-=-=-=-

#declare FractalType = 3;
#declare ColorMethod = 2;
#declare UsePalette  = true;
#declare NIterations = 256;

#declare ShowNumbers = true;
#declare DisplayGrid = true;
#declare GuideColors = rgb<1,1,1>;

// navigate through the fractal here
#macro Navigation()
 
 Zoom_Region(<-.125,.4,0>, 0.25, -30, 0)
 
#end

// -=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-

 // this calculates the fractal and returns a function
#macro iterate(iters)
 
 /* Interior coloring. You can change '0' to whatever function you want. */
 #local iterate_1 = function(x,y,PX,PY){ 0 }
 
 #local i = 0;
 #while(i<iters)
  #local iterate_2 = iterate_1; #undef iterate_1
  #local iterate_1 =
   function(x,y,PX,PY){
    select(
     
     // testing against the magnitude
     // '-20' is best for normalized (log) smoothing
     sqrt(x*x+y*y)-20,
     
     iterate_2(
      
      // formulae go here
      #switch(FractalType)
       
       // BurningBrot var 2, with options
       #case(9)
        #local influence = -0.25; // <-- change this
        max(x*x-abs(y)*abs(y)+influence,x*x-y*y) +PX,
			  max(x*abs(y)*2+influence,x*y*2)          +PY,
       #break
       
       // BurningBrot, with options - keep influence < 0
       #case(8)
        #local influence = -0.05; // <-- change this
        #local rot = 0;
        min(abs(x)*abs(x)-abs(y)*abs(y)+influence,x*x-y*y) +PX,
			  min(abs(x)*abs(y)*2+influence,x*y*2)               +PY,
       #break
       
       // half Burning Ship
       #case(7)
        x*x-abs(y)*abs(y) +PX,
        x*abs(y)*2        +PY,
       #break
       
       // MSet variation 3
       #case(6)
        min(abs(x*x-y*y), abs(.5-x*x+y*y)) +PX,
			  min(abs(x*y*2), abs(.5+x*-y*2))    +PY,
       #break
       
       // MSet variation 1
       #case(5)
        min((x*x)-(y*y),.65+((-x)*(y))*2) +PX,
        min((x*y)*2,.65-((x*x)-(y*y)))    +PY,
       #break
       
       // Burning Ship
       #case(4)
        abs(x)*abs(x)-abs(y)*abs(y) +PX,
        abs(x)*abs(y)*2             +PY,
       #break
       
       // Glynn Fractal, with options (normalized smoothing doesn't work well here)
       #case(3)
        #local Power    = 1.5;     // <-- change these
        #local PerturbX = -0.19;  // <-'
        pow(sqrt(x*x+y*y),Power)*cos(atan2(y,x)*Power) +PerturbX,
        pow(sqrt(x*x+y*y),Power)*sin(atan2(y,x)*Power),
       #break
       
       // Julia, with options
       #case(2)
        #local PerturbX = -0.7;  // <-- change these
        #local PerturbY = -0.3; // <-'
        x*x-y*y +PerturbX,  
        x*y*2   +PerturbY,
       #break
       
       // classic Mandelbrot
       #case(1)
        x*x-y*y +PX,  
        x*y*2   +PY,
       #break
       
       #case(0)
        0,
        0,
       #break
       
      #end
      
      PX, PY
     ),
     
     // exterior coloring schemes
     #switch(ColorMethod)
      
      #case(0)
       0
      #break
      
      #case(1)  //  by iteration (basic)
       1-(i-1)/iters
      #break
      
      #case(2) /* normalized smoothing, great for the M-Set, Julia (ref: http://linas.org/art-gallery/escape/escape.html) */
       i-(log(log(sqrt(x*x+y*y))))/log(2.0)/iters-i/iters
      #break
      
     #end
    )
   }
  #undef iterate_2
  #local i=i+1;
 #end
 function{ iterate_1( x, y, x, y )#if(ColorMethod=1)*iters/256#end}
 #undef iterate_1
#end // iterate

global_settings{assumed_gamma 1.0}

// some helper macros
#declare AspectX = image_width/image_height;
#declare LThk = 1.25/max(AspectX,image_height);
#macro Zoom_Box(Position, Scale, Rotation)
 union{
  cylinder{Scale*<-AspectX,-1,0>,Scale*<-AspectX,1,0>,LThk}
  cylinder{Scale*<AspectX,-1,0>,Scale*<AspectX,1,0>,LThk}
  cylinder{Scale*<-AspectX,-1,0>,Scale*<AspectX,-1,0>,LThk}
  cylinder{Scale*<-AspectX,1,0>,Scale*<AspectX,1,0>,LThk}
  rotate -z*Rotation
  translate Position
  pigment{GuideColors}
  finish{ambient 1}
 }
#end
#declare View_Zoom_Box = false;
#declare ZBPosition = <0,0>;
#declare ZBScale = 1;
#declare ZBRotation = 0;
#macro Zoom_Region(Position, Scale, Rotation, DoZoom)
 #local Position = Position*<AspectX,1,1>;
 #declare ZBPosition=Position;
 #declare ZBScale=Scale;
 #declare ZBRotation=Rotation;
 #if(DoZoom)
  #declare View_Zoom_Box = false;
  translate -Position
  rotate z*Rotation
  scale 1/Scale
 #else
  #declare View_Zoom_Box = true;
 #end
#end // macros

// the visible plane
plane{z,0
 pigment{
  
  /* Calling The Fractal
     (try not use use more than 1023 iterations) */
  iterate(NIterations)
  
  // corrective scaling
  scale<1,-1,1>
  
  Navigation()
  
  // Color Control
  phase 0
  frequency 1
  ramp_wave
  
  /*
   Palette Support
   
   Available palettes:
     chroma / default / evening / golden / greens
     metallic / mushroom / organic / shadows / variations
  */
  #if(UsePalette)
   // change the palette here
   #declare img   = pigment{image_map{png"palettes/evening.png" interpolate 0}}
   #declare img_w = max_extent(img).x;
   #declare f_img = function{pigment{img}}
   color_map{
    [0 rgb 0]
    #local V=0;
    #while(V<min(255,img_w))
     #local X = 1-V/img_w+.5/img_w;
     [V/img_w
      rgb<f_img(X,0,0).x,f_img(X,0,0).y,f_img(X,0,0).z>
     ]
     #local V=V+1;
    #end
   }
  #end
  
 }
 finish{ambient 1.3}
}

// visible zoom box
#if(View_Zoom_Box)
 Zoom_Box(ZBPosition, ZBScale, ZBRotation)
#end

// grid & numbers
#if(View_Zoom_Box)
 union{
  #declare V = 0;
  #while(V<=1)
   #if(V>0)
    #if(V<1 & DisplayGrid)
     union{
      cylinder{-x*AspectX,x*AspectX,LThk translate y*(-1+2*V)}
      cylinder{-y,y,LThk translate x*(-1+2*V)*AspectX}
      pigment{GuideColors transmit .75}
      finish{ambient 1}
     }
    #end
   #end
   #if(ShowNumbers)
    #if(V>0)
     text{ttf"cyrvetic.ttf" str(-1+V*2,0,2) 1 0 translate -z-y/2.5*(1+(V=1)) scale 24*LThk translate -x*AspectX+y*(-1+2*V) }
    #end
    text{ttf"cyrvetic.ttf" str(-1+V*2,0,2) 1 0 translate -x*(1*(V>0)+(V=1))-z scale 24*LThk translate -y+x*(-1+2*V)*AspectX }
   #end
   #declare V=V+1/8;
  #end
  pigment{GuideColors}
  finish{ambient 1}
 }
#end

camera{
 orthographic
 right x*AspectX*2 up y*2
 location -z
}